/***************************************************************************
                          colorDg.cpp  -  description
                             ---------------------
		Class that handles the Option->color dialog.

        Version         : $Id: colorDg.cpp,v 1.4 2000/05/15 09:25:09 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qframe.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qgrpbox.h>
#include <qlayout.h>
#include <qcolor.h>

// Kde includes
#include <kapp.h>
#include <kcolorbtn.h>
#include <klocale.h>

// Application specific includes
#include "kmconfig.h"
#include "colorDg.h"
#include "kmlog.h"

extern kmConfig *kmergeCfg;

#define logLL

QPalette & newPalette( const QColor & fg, const QColor & bg, const QPalette & qp)
{
	QColorGroup oldCg;
	QPalette * newQp;
	
	oldCg = qp.normal();
	QColorGroup cg( oldCg.foreground(), bg, oldCg.light(), oldCg.dark(), 
			oldCg.midlight(), fg, oldCg.base() );	
	newQp = new QPalette( cg, qp.disabled(), qp.active() );
	CHECK_PTR( newQp );
	
	return *newQp;
}


kmColorDg::kmColorDg(QWidget * parent)
:QDialog(parent, "colorDialog", true)
{
	QBoxLayout *vl = new QVBoxLayout(this, 5);

	QGridLayout *gl = new QGridLayout(5, 3, 10);
	vl->addLayout(gl);

	QSize sz;
	QFont lFont(font());
	lFont.setBold(true);

	QLabel *lab;
	lab = new QLabel(this);
	CHECK_PTR(lab);
	lab->setText(i18n("Foreground:"));
	lab->setFont(lFont);
	lab->setAlignment(AlignHCenter | AlignVCenter);
	lab->setMinimumSize(lab->sizeHint());
	gl->addWidget(lab, 0, 2);
	sz = lab->sizeHint();

	lab = new QLabel(this);
	CHECK_PTR(lab);
	lab->setText(i18n("Background:"));
	lab->setFont(lFont);
	lab->setAlignment(AlignHCenter | AlignVCenter);
	lab->setMinimumSize(lab->sizeHint());
	gl->addWidget(lab, 0, 1);
	sz.setWidth(QMAX(sz.width(), lab->sizeHint().width()));

	sz.setWidth(sz.width() - 10);

	QFont font(kmergeCfg->font);

	labN = new QLabel(this);
	CHECK_PTR(labN);
	labN->setText(i18n("Normal lines"));
	labN->setFont(font);
	labN->setAlignment(AlignHCenter | AlignVCenter);
      labN->setFrameStyle(QFrame::Panel | QFrame::Plain);
	labN->setMinimumSize(lab->sizeHint());
	gl->addWidget(labN, 1, 0);
	sz.setHeight(labN->sizeHint().height() - 5);

	labA = new QLabel(this);
	CHECK_PTR(labA);
	labA->setText(i18n("Added lines"));
	labA->setFont(font);
	labA->setAlignment(AlignHCenter | AlignVCenter);
      labA->setFrameStyle(QFrame::Panel | QFrame::Plain);
	labA->setMinimumSize(lab->sizeHint());
	gl->addWidget(labA, 2, 0);

	labD = new QLabel(this);
	CHECK_PTR(labD);
	labD->setText(i18n("Deleted lines"));
	labD->setFont(font);
	labD->setAlignment(AlignHCenter | AlignVCenter);
      labD->setFrameStyle(QFrame::Panel | QFrame::Plain);
	labD->setMinimumSize(lab->sizeHint());
	gl->addWidget(labD, 3, 0);

	labC = new QLabel(this);
	CHECK_PTR(labC);
	labC->setText(i18n("Changed lines"));
	labC->setFont(font);
	labC->setAlignment(AlignHCenter | AlignVCenter);
	labC->setFrameStyle(QFrame::Panel | QFrame::Plain);
	labC->setMinimumSize(lab->sizeHint());
	gl->addWidget(labC, 4, 0);

	sz = QSize(80, 30);
	norFgColBt = new KColorButton(kmergeCfg->norFgCol, this);
	norFgColBt->setMinimumSize(norFgColBt->sizeHint());
	norFgColBt->setFixedSize(sz);
	gl->addWidget(norFgColBt, 1, 2, AlignHCenter | AlignVCenter);

	norBgColBt = new KColorButton(kmergeCfg->norBgCol, this);
	norBgColBt->setMinimumSize(norBgColBt->sizeHint());
	norBgColBt->setFixedSize(sz);
	gl->addWidget(norBgColBt, 1, 1, AlignHCenter | AlignVCenter);

	addFgColBt = new KColorButton(kmergeCfg->addFgCol, this);
	addFgColBt->setMinimumSize(addFgColBt->sizeHint());
	addFgColBt->setFixedSize(sz);
	gl->addWidget(addFgColBt, 2, 2, AlignHCenter | AlignVCenter);

	addBgColBt = new KColorButton(kmergeCfg->addBgCol, this);
	addBgColBt->setMinimumSize(addBgColBt->sizeHint());
	addBgColBt->setFixedSize(sz);
	gl->addWidget(addBgColBt, 2, 1, AlignHCenter | AlignVCenter);

	delFgColBt = new KColorButton(kmergeCfg->delFgCol, this);
	delFgColBt->setMinimumSize(delFgColBt->sizeHint());
	delFgColBt->setFixedSize(sz);
	gl->addWidget(delFgColBt, 3, 2, AlignHCenter | AlignVCenter);

	delBgColBt = new KColorButton(kmergeCfg->delBgCol, this);
	delBgColBt->setMinimumSize(delBgColBt->sizeHint());
	delBgColBt->setFixedSize(sz);
	gl->addWidget(delBgColBt, 3, 1, AlignHCenter | AlignVCenter);

	chgFgColBt = new KColorButton(kmergeCfg->chgFgCol, this);
	chgFgColBt->setMinimumSize(chgFgColBt->sizeHint());
	chgFgColBt->setFixedSize(sz);
	gl->addWidget(chgFgColBt, 4, 2, AlignHCenter | AlignVCenter);

	chgBgColBt = new KColorButton(kmergeCfg->chgBgCol, this);
	chgBgColBt->setMinimumSize(chgBgColBt->sizeHint());
	chgBgColBt->setFixedSize(sz);
	gl->addWidget(chgBgColBt, 4, 1, AlignHCenter | AlignVCenter);

	norColChanged();
	addColChanged();
	delColChanged();
	chgColChanged();

	connect(norFgColBt, SIGNAL(changed(const QColor &)),
		SLOT(norColChanged()));
	connect(norBgColBt, SIGNAL(changed(const QColor &)),
		SLOT(norColChanged()));
	connect(addFgColBt, SIGNAL(changed(const QColor &)),
		SLOT(addColChanged()));
	connect(addBgColBt, SIGNAL(changed(const QColor &)),
		SLOT(addColChanged()));
	connect(delFgColBt, SIGNAL(changed(const QColor &)),
		SLOT(delColChanged()));
	connect(delBgColBt, SIGNAL(changed(const QColor &)),
		SLOT(delColChanged()));
	connect(chgFgColBt, SIGNAL(changed(const QColor &)),
		SLOT(chgColChanged()));
	connect(chgBgColBt, SIGNAL(changed(const QColor &)),
		SLOT(chgColChanged()));

	int i;
	for (i = 0; i < gl->numRows(); i++)
		gl->setRowStretch(i, 1);

	for (i = 1; i < gl->numCols(); i++)
		gl->setColStretch(i, 1);
	gl->setColStretch(0, 2);

//gl->addColSpacing( 1, sz.width() );
	//gl->addColSpacing( 2, sz.width() );

	vl->addSpacing( 10 );

	QBoxLayout *hl = new QHBoxLayout(5);
	vl->addLayout(hl);

	QPushButton *tmpQPushB;
	tmpQPushB = new QPushButton(this, "okBtn");
	tmpQPushB->setFixedSize(70, 25);
	connect(tmpQPushB, SIGNAL(clicked()), this, SLOT(ok()));
	tmpQPushB->setText(i18n("Ok"));
	tmpQPushB->setAutoDefault(true);
	hl->addWidget(tmpQPushB, 1);

	tmpQPushB = new QPushButton(this);
	tmpQPushB->setFixedSize(70, 25);
	connect(tmpQPushB, SIGNAL(clicked()), this, SLOT(defaultCol()));
	tmpQPushB->setText(i18n("Default"));
	hl->addWidget(tmpQPushB, 1);

	tmpQPushB = new QPushButton(this, "cancelBtn");
	tmpQPushB->setFixedSize(70, 25);
	connect(tmpQPushB, SIGNAL(clicked()), this, SLOT(cancel()));
	tmpQPushB->setText(i18n("Cancel"));
	hl->addWidget(tmpQPushB, 1);

	hl->addStretch(10);

	tmpQPushB = new QPushButton(this, "helpBtn");
	tmpQPushB->setFixedSize(70, 25);
	connect(tmpQPushB, SIGNAL(clicked()), this, SLOT(help()));
	tmpQPushB->setText(i18n("Help"));
	hl->addWidget(tmpQPushB, 1);

	vl->activate();

	resize( 354, 220 );

	setCaption(i18n("Kmerge Colors Setup"));
}

void kmColorDg::norColChanged()
{
	labN->setPalette( newPalette( norFgColBt->color(), norBgColBt->color(),
				   labN->palette() ) );
	QDialog::update();
}

void kmColorDg::addColChanged()
{
	labA->setPalette( newPalette( addFgColBt->color(), addBgColBt->color(),
				   labA->palette() ) );
	QDialog::update();
}


void kmColorDg::delColChanged()
{
	labD->setPalette( newPalette( delFgColBt->color(), 
				      delBgColBt->color(),
				      labD->palette()      ) );
	QDialog::update();
}


void kmColorDg::chgColChanged()
{
	labC->setPalette( newPalette( chgFgColBt->color(), 
				      chgBgColBt->color(),
				      labC->palette()     ) );
	QDialog::update();
}

void
 kmColorDg::defaultCol()
{
	norFgColBt->setColor(black);
	norBgColBt->setColor(white);
	addFgColBt->setColor(black);
	addBgColBt->setColor(green);
	delFgColBt->setColor(black);
	delBgColBt->setColor(cyan);
	chgFgColBt->setColor(black);
	chgBgColBt->setColor(yellow);
	
	norColChanged();
	addColChanged();
	delColChanged();
	chgColChanged();
}

kmColorDg::~kmColorDg()
{
// I love Qt for these empty destructors
}

void kmColorDg::ok()
{
	kmergeCfg->norFgCol = norFgColBt->color();
	kmergeCfg->norBgCol = norBgColBt->color();
	kmergeCfg->addFgCol = addFgColBt->color();
	kmergeCfg->addBgCol = addBgColBt->color();
	kmergeCfg->chgFgCol = chgFgColBt->color();
	kmergeCfg->chgBgCol = chgBgColBt->color();
	kmergeCfg->delFgCol = delFgColBt->color();
	kmergeCfg->delBgCol = delBgColBt->color();

	done(1);
}

void kmColorDg::cancel()
{
	done(0);
}

void kmColorDg::help()
{
	QString filename("/kdiff-4.html");

// kapp is defined in kapp.h as #define kapp KApplication::getKApplication()
	kapp->invokeHTMLHelp(filename, "ss4.2");

}

void kmColorDg::resizeEvent(QResizeEvent * re)
{
	QDialog::resizeEvent(re);
}
