/***************************************************************************
                          diffIO.cpp  -  description
                             --------------------

		This class handles all interaction with the diff 
		program (launch and parse output).

        Version         : $Id: diffIO.cpp,v 1.10 2000/05/15 09:25:09 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// System includes
#include <ctype.h>
#include <iostream.h>

// Qt includes
#include <qfile.h>
#include <qtextstream.h>

// Application specific includes
#include "diffIO.h"
#include "kmconfig.h"
#include "kmlog.h"

extern kmConfig *kmergeCfg;

//#define logLL logDD
#define logLL(args...) 

/*
 * 	Be careful and remember, ** first line is line 1 **
 * 	leftS is 1 for the very first line
 */

/*                     ---------------
 *                        kLineDiff
 *                     ---------------                  */                     
kLineDiff::kLineDiff( char * arg1, char * arg2, kLineDiff * lastDiff = 0)
{
	int len, nbRec;
	
	leftS = leftE = rightS = rightE = 0;
	dispLine = -1;
	curFileSize = diffSize = 0;
	
        nbRec = sscanf(arg1, "-%d,%d", &leftS, &len);

        if (nbRec == 0)
                logEE( "parsing diff output: '%s' not understood!\n", arg1);
        else if (nbRec == 1)
                len = 1;

	// Need to handle the case where diff reports a size of 0 separately.
	if (len == 0)
		leftS++;
	leftE = leftS + len;

        nbRec = sscanf(arg2, "+%d,%d", &rightS, &len);

        if (nbRec == 0)
                logEE( "parsing diff output: '%s' not understood!\n", arg2);
        else if (nbRec == 1)
                len = 1;

	// Need to handle the case where diff reports a size of 0 separately.
	if (len == 0)
		rightS++;
	rightE = rightS + len;

	if (leftE == leftS) {
		if (rightE == rightS) {
			logWW("parseDiffOutput : no difference found!\n");
			type = NoDiff;
		}
		else
			type = Added;
	}
	else {
		if (rightE == rightS) 
			type = Deleted;
		else
			type = Changed;
	}
	logLL( "recognized difference: %d - %d , %d - %d", leftS, leftE, rightS,
			rightE );

	diffSize = QMAX( leftE - leftS, rightE - rightS );

	if (lastDiff == 0) {
		// This is the first difference
		kmAssert( leftS == rightS, "The first difference has different left & right line number: %d vs %d", leftS, rightS);
		//logLL("First diff found! %d", leftS);
		
		// This the number of lines the files contains before the
		// difference
		curFileSize = leftS-1;

	} else {
		kmAssert( (leftS-lastDiff->leftE) == (rightS-lastDiff->rightE),
			"This difference has different left & right normal line size: %d vs %d", (leftS - lastDiff->leftE) ,(rightS - lastDiff->rightE));

		curFileSize = lastDiff->curFileSize + lastDiff->diffSize;
		curFileSize += rightS - lastDiff->rightE;
	}

	//logLL("This diff: size = %d, file size so far = %d", diffSize, curFileSize);

	
}


kLineDiff::~kLineDiff()
{
}

/*                     ---------------
 *                        kFileDiff
 *                     ---------------                  */                     
kFileDiff::kFileDiff()
{
	diffList.setAutoDelete( TRUE );
	leftStrings.setAutoDelete( TRUE );
	rightStrings.setAutoDelete( TRUE );

	leftStrings.clear();
	rightStrings.clear();
	diffList.clear();

	leftFileName="";
	rightFileName="";

	leftLongestLine = rightLongestLine = -1;
}

kFileDiff::~kFileDiff()
{
	leftStrings.clear();
	rightStrings.clear();
	diffList.clear();
}
	 

int
kFileDiff::loadLeftFile( QString lFile )
{
	//logLL("Opening left file : %s", (const char *) lFile);
	
	//if (! leftStrings.isEmpty() ) {
		leftStrings.clear();
		leftFileName = "";
	//}
		
	uint maxCh;
	QFile f(lFile);
	if ( ! f.open(IO_ReadOnly) ) {
		return -1;
	}
    
	QTextStream ts( &f );        // use a text stream        
	QString s;        
	
	leftLongestLine = -1;
	maxCh=0;
	while ( !ts.eof() ) {        
		// until end of file...            
		s = ts.readLine();      
		//logLL("Reading left file: %s", (const char *) s);
		leftStrings.append( s );
		
		// We need to know the size of the longest string	
		if ( maxCh < s.length() ) {
			maxCh = s.length();
			leftLongestLine = leftStrings.at();
		}	
	}        
	f.close();
	
	diffList.clear();

	//logLL(": %d left strings read", leftStrings.count() );
	//logLL(": longest line is line: %d", leftLongestLine);
	
	if ( leftStrings.isEmpty() ) {
		return -2;
	}

	leftFileName = lFile;

	emit leftFileChanged();

	return 0;
}

int
kFileDiff::loadRightFile( QString rFile )
{
	//logLL("Opening right file : %s", (const char *) rFile);
	
//	if (! rightStrings.isEmpty() ) {
		rightStrings.clear();
		rightFileName = "";
//	}
		
	uint maxCh;
	QFile f(rFile);
	if ( ! f.open(IO_ReadOnly) ) {
		return -1;
	}
    
	QTextStream ts( &f );        // use a text stream        
	QString s;        

	maxCh = 0;	
	rightLongestLine = -1;
	while ( !ts.eof() ) {        
		// until end of file...            
		s = ts.readLine();      
		//logLL("Reading right file: %s", (const char *) s);
		rightStrings.append( s );
		
		// We need to know the size of the longest string	
		if ( maxCh < s.length() ) {
			maxCh = s.length();
			rightLongestLine = rightStrings.at();
		}	
	}        
	f.close();    
	diffList.clear();
	
	//rTitle->setText( rFile.data() );

	//logLL(": %d right strings read", rightStrings.count() );
	//logLL(": longest line is line: %d", rightLongestLine);
	
	if ( rightStrings.isEmpty() ) {
		return -2;
	}

	rightFileName = rFile;

	emit rightFileChanged();

	return 0;
}


/*                     ---------------
 *                        kDiffIO
 *                     ---------------                  */                     
int
kDiffIO::parseDiffOutput(kFileDiff * fileDiff)
{
	char * curLine;
	char str1[1024];	
	char str2[1024];
	int nbRec;
	kLineDiff *lineDiff=0;
	kLineDiff *lastLine=0;

	curLine = strtok( diffStdoutStr.data(), "\n\0" );
	while ( curLine != 0 ) {

		logLL( "Treating : %s", curLine );

		// Left File
		nbRec = sscanf(curLine, "--- %s", str1);
		if (nbRec == 1) {
			if (! fileDiff->leftFileName.contains(str1)) {
				logWW("kFileDiff left file name '%s' differs with diff left file name '%s'", (const char *) fileDiff->leftFileName, str1);
			}
		}
		
		// Right File
		nbRec = sscanf(curLine, "+++ %s", str2);
		if (nbRec == 1 ) {
			if (! fileDiff->rightFileName.contains(str2)) {
				logWW("kFileDiff right file name '%s' differs with diff right file name '%s'", (const char *) fileDiff->rightFileName, str2);
			}
		}

		// A diff line
		nbRec = sscanf( curLine, "@@ %s %s @@", str1, str2 );
		if ( nbRec == 2 ) {
			lineDiff = new kLineDiff( str1, str2, lastLine );
			fileDiff->diffList.append( lineDiff );
			lastLine = lineDiff;
		}

		curLine = strtok( 0, "\n\0" );
	}

	return 0;
}
			
   	
kDiffIO::kDiffIO()
{
	diffStdoutStr = QString();
	diffStderrStr = QString();
	diffProcess = 0;
}

kDiffIO::~kDiffIO()
{
	delete diffProcess;
}

void 
kDiffIO::diffStdout(KProcess * proc, char *buffer, int len)
{
	// It seems that buffer is NOT null terminated and contains exactly
	// len characters.
        
	char * chTab = new char[len+1];
	strncpy(chTab, buffer, len);
	chTab[len] = 0;

	logLL(": [[%s]]", chTab);
	
	kmAssert( (int) strlen(chTab) == len, "non null-terminated buffer: len = %d, strlen returns %d", len, strlen(chTab));	

        diffStdoutStr += chTab;
	
	delete [] chTab;
}

void 
kDiffIO::diffStderr(KProcess * proc, char *buffer, int len)
{
	char * chTab = new char[len+1];
	strncpy(chTab, buffer, len);
	chTab[len] = 0;

	logLL(": [[%s]]", chTab);
	
	kmAssert( (int) strlen(chTab) == len, "non null-terminated buffer: len = %d, strlen returns %d", len, strlen(chTab));	

        diffStderrStr += chTab;
	
	delete [] chTab;
}

int kDiffIO::runDiff(kFileDiff * fileDiff)
{
        int retValue = 0;

	diffStdoutStr = "";
	diffStderrStr = "";

	//diffProcess->clearArguments(); since snapshot 01/18/98 it
	//doesn't work
	// so i create diffProcess new every time a diff-process is
	// needed
	if (diffProcess != 0 ) {
		delete diffProcess;
		diffProcess = 0;
	}

	diffProcess = new KProcess();
	
	// '\diff' to remove the alias of the user
	//diffProcess->setExecutable("\diff");
	diffProcess->setExecutable("diff");
	*diffProcess << "-u" << "-U" << "0";

	// diff options:
	
	if ( kmergeCfg->ignoreSpaces )
		*diffProcess << "--ignore-space-change";
	
	if ( kmergeCfg->ignoreAllSpaces )
		*diffProcess << "--ignore-all-space";

	if ( kmergeCfg->ignoreBlankLines )
		*diffProcess << "--ignore-blank-lines";

	if ( kmergeCfg->ignoreCase )
		*diffProcess << "--ignore-case";
                	
	if ( kmergeCfg->ignoreRegExpLines
		&& ! kmergeCfg->ignoreRegExp.isEmpty() ) {
		*diffProcess << "--ignore-matching-lines=";
		*diffProcess << kmergeCfg->ignoreRegExp;
	}
	
	if ( kmergeCfg->speedLargeFiles )
		*diffProcess << "--speed-large-files";

	if ( kmergeCfg->minimalDifferences )
		*diffProcess << "--minimal";
	
	if ( kmergeCfg->customizeDiff && ! kmergeCfg->customExp.isEmpty() )
		*diffProcess << kmergeCfg->customExp;
	
	logLL("diffing files: '%s' against '%s'", 
		(const char *) fileDiff->leftFileName, 
		(const char *) fileDiff->rightFileName);
	*diffProcess << fileDiff->leftFileName << fileDiff->rightFileName;

	connect( diffProcess, SIGNAL(receivedStdout(KProcess *, char *,
		int)), SLOT(diffStdout(KProcess *, char *, int)));
	connect( diffProcess, SIGNAL(receivedStderr(KProcess *, char *,
		int)), SLOT(diffStderr(KProcess *, char *, int)));
	
	//connect( diffProcess, SIGNAL(processExited(KProcess*)),
	//SLOT(diffReady(KProcess*)) );

	bool a = diffProcess->start(KProcess::Block, 
				/*NotifyOnExit, */
				KProcess::AllOutput);
	
	if (a != true || diffStderrStr.length() > 0) {
		retValue = -1;
	}

	logII("Exitstatus: %d", diffProcess->exitStatus());

	retValue = parseDiffOutput(fileDiff);
	
	return (retValue);
}

QString 
kDiffIO::getDiffStderr()
{
	return diffStderrStr;
}


