/***************************************************************************
                          diffIO.h  -  description
                             -------------------

		Interaction with diff program, parse of output.

        Version         : $Id: diffIO.h,v 1.6 2000/04/30 22:03:33 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DIFFIO_H
#define DIFFIO_H

// Qt include files
#include <qobject.h>
//#include <qwidget.h>
#include <qstring.h>

// Kde include files
//#include <kurl.h>
#include <kprocess.h>

enum DiffType {
	NoDiff,
	Added,
	Deleted,
	Changed
};

/*                     ---------------
 *                        kLineDiff
 *                     ---------------                  */                     
class kLineDiff 
{
	public:
		kLineDiff( char * arg1, char * arg2, kLineDiff * lastDiff = 0);
		~kLineDiff();
		  
		DiffType type;
		int leftS, leftE, rightS, rightE;
		int dispLine;
		int diffSize, curFileSize;
};

/*                     ---------------
 *                        kFileDiff
 *                     ---------------                  */                     
class kFileDiff :  public QObject
{
		Q_OBJECT
	public:

		kFileDiff();
		~kFileDiff();

		QStrList leftStrings;
		QStrList rightStrings;

		QString leftFileName;
		QString rightFileName;

		int leftLongestLine;
		int rightLongestLine;
		
		int loadLeftFile( QString lFile );
		int loadRightFile( QString rFile );

		QList<kLineDiff> diffList;

	signals:
		void leftFileChanged();
		void rightFileChanged();
		void diffAvailable();
};

/*                     ---------------
 *                        kDiffIO
 *                     ---------------                  */                     
class kDiffIO : public QObject
{
	Q_OBJECT;

	public:
		kDiffIO();
		~kDiffIO();

		int runDiff(kFileDiff * fileDiff);
		QString getDiffStderr();

	public slots:
		void diffStdout( KProcess*, char*, int );
		void diffStderr( KProcess*, char*, int );
		
	signals:
		void processState( int );

	protected:
		int parseDiffOutput(kFileDiff * fileDiff);
		
		KProcess *diffProcess;             // Instance to start diff

		QString diffStdoutStr;             // STDOUT from diff-process
		QString diffStderrStr;             // STDERR from diff-process
	  
};

#endif // DIFFIO_H


