/***************************************************************************
                          kmerge.cpp  -  description
                             -------------------

			The main class that creates everything

        Version         : $Id: kmerge.cpp,v 1.12 2000/05/16 12:18:12 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// System includes
#include <iostream.h>
// Qt includes
#include <qcolor.h>
// Kde includes
#include <kstdaccel.h>
#include <kiconloader.h>
#include <kfontdialog.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kconfig.h>

// Application specific includes
#include "kmerge.h"
#include "kmwidget.h"
#include "diffIO.h"
#include "diffDg.h"
#include "colorDg.h"
#include "mergeDg.h"
#include "kmlog.h"

// Toolbar button Id
#define TB_OPENLEFT	1
#define TB_OPENRIGHT	2
#define TB_DODIFF	3
#define TB_PREVDIFF	4
#define TB_NEXTDIFF	5
#define TB_MERGEMODE	6
#define TB_TAKELEFT	7
#define TB_TAKERIGHT	8
#define TB_UNSELECTDIFF	9

kmConfig * kmergeCfg;


Kmerge::Kmerge( const char *name)
	: KTMainWindow(name)
{
	//setLogFile( "/tmp/kmerge.log");

	kmergeCfg = new kmConfig;


	diffIO = new kDiffIO();
	fileDiff = new kFileDiff();
	
	initConfig();

	kmw = new kmWidget(this);
	setView(kmw);
	kmw->setDiff( fileDiff );

	initMenuBar();
	initToolBar();
	//initStatusBar();

	connect( kmw, SIGNAL(sg_mergeComplete()), this, SLOT( mergeComplete()));

	kmergeState( KM_NoFile );

	/*	
	for( int i=0; i<kapp->argc(); i++) {
		logDD( "argv[ %d ] : %s", i, kapp->argv()[i] );
	}
	*/
   
	if (kapp->argc() > 1) {
		leftFile = kapp->argv()[1];
		fileDiff->loadLeftFile( leftFile );
		kmw->leftFileChanged();
	}

	if (kapp->argc() > 2) {
		rightFile = kapp->argv()[2];
		fileDiff->loadRightFile( rightFile );
		kmw->rightFileChanged();
		doDiff();
	}
	
}

void Kmerge::initVar()
{
}


Kmerge::~Kmerge()
{
	delete kmergeCfg;
	delete diffIO;
	delete fileDiff;

	/*
	delete toolBar;
	delete statusBar;
	delete menuBar;
	*/
}

void Kmerge::mergeMode()
{
	if (kmw->mergeMode()) {
		kmergeState( KM_DispDiff );
	} else {
		kmergeState( KM_Merge );
	}
}

void Kmerge::mergeComplete()
{
	kmw->setMergeMode( false );
	kmergeState( KM_DispDiff );
}

void Kmerge::fileJustLoaded()
{
	if (! fileDiff->leftFileName.isEmpty()
	    && ! fileDiff->rightFileName.isEmpty()) 
		kmergeState( KM_DispFile );
	else kmergeState( KM_NoFile );
}

void Kmerge::kmergeState( KmState status )
{
	switch( status ) {

	case KM_NoFile:
		toolBar->setItemEnabled( TB_DODIFF, false );
		toolBar->setItemEnabled( TB_PREVDIFF, false );
		toolBar->setItemEnabled( TB_NEXTDIFF, false );
		toolBar->setItemEnabled( TB_TAKELEFT, false);
		toolBar->setItemEnabled( TB_TAKERIGHT, false);
		toolBar->setItemEnabled( TB_MERGEMODE, false);
		diffPm->setItemEnabled( TB_PREVDIFF, false );
		diffPm->setItemEnabled( TB_NEXTDIFF, false );
		diffPm->setItemEnabled( TB_DODIFF, false );
		diffPm->setItemEnabled( TB_UNSELECTDIFF, false );
		mergePm->setItemEnabled( TB_TAKELEFT, false);
		mergePm->setItemEnabled( TB_TAKERIGHT, false);
		mergePm->setItemEnabled( TB_MERGEMODE, false);
		kmw->setMergeMode( false );
		break;

	case KM_DispFile:
		toolBar->setItemEnabled( TB_DODIFF, true );
		toolBar->setItemEnabled( TB_PREVDIFF, false );
		toolBar->setItemEnabled( TB_NEXTDIFF, false );
		toolBar->setItemEnabled( TB_TAKELEFT, false);
		toolBar->setItemEnabled( TB_TAKERIGHT, false);
		toolBar->setItemEnabled( TB_MERGEMODE, false);
		diffPm->setItemEnabled( TB_DODIFF, true );
		diffPm->setItemEnabled( TB_PREVDIFF, false );
		diffPm->setItemEnabled( TB_NEXTDIFF, false );
		diffPm->setItemEnabled( TB_UNSELECTDIFF, false );
		mergePm->setItemEnabled( TB_TAKELEFT, false);
		mergePm->setItemEnabled( TB_TAKERIGHT, false);
		mergePm->setItemEnabled( TB_MERGEMODE, false);
		kmw->setMergeMode( false );
		break;

	case KM_DispDiff:
		toolBar->setItemEnabled( TB_DODIFF, false);
		toolBar->setItemEnabled( TB_PREVDIFF, true );
		toolBar->setItemEnabled( TB_NEXTDIFF, true );
		toolBar->setItemEnabled( TB_TAKELEFT, false);
		toolBar->setItemEnabled( TB_TAKERIGHT, false);
		toolBar->setItemEnabled( TB_MERGEMODE, false);
		diffPm->setItemEnabled( TB_DODIFF, false);
		diffPm->setItemEnabled( TB_PREVDIFF, true );
		diffPm->setItemEnabled( TB_NEXTDIFF, true );
		diffPm->setItemEnabled( TB_UNSELECTDIFF, true );
		mergePm->setItemEnabled( TB_TAKELEFT, false);
		mergePm->setItemEnabled( TB_TAKERIGHT, false);
		mergePm->setItemEnabled( TB_MERGEMODE, true);
		kmw->setMergeMode( false );
		break;

	case KM_Merge:
		toolBar->setItemEnabled( TB_DODIFF, false);
		toolBar->setItemEnabled( TB_PREVDIFF, false );
		toolBar->setItemEnabled( TB_NEXTDIFF, false );
		toolBar->setItemEnabled( TB_TAKELEFT, true);
		toolBar->setItemEnabled( TB_TAKERIGHT, true);
		toolBar->setItemEnabled( TB_MERGEMODE, true);
		diffPm->setItemEnabled( TB_DODIFF, false);
		diffPm->setItemEnabled( TB_PREVDIFF, false );
		diffPm->setItemEnabled( TB_NEXTDIFF, false );
		diffPm->setItemEnabled( TB_UNSELECTDIFF, false );
		mergePm->setItemEnabled( TB_TAKELEFT, true);
		mergePm->setItemEnabled( TB_TAKERIGHT, true);
		mergePm->setItemEnabled( TB_MERGEMODE, true);
		kmw->setMergeMode( true );
		break;

	}
}


void
Kmerge::openLeft()
{
	if ( leftFileDir.isEmpty() )
		if ( ! rightFileDir.isEmpty() )
			leftFileDir = rightFileDir.copy();

  	KFileDialog fdg( (const char *) leftFileDir, 0, 
		this, "lopen", true, false );
	fdg.setCaption( i18n( "Open left file") );
	
	if( fdg.exec() && !fdg.selectedFile().isEmpty() ) {
		//logDD("Left file selected: %s", fdg.selectedFile().data());
		leftFileDir = fdg.dirPath();
		leftFile = fdg.selectedFile().copy();
		if (! fileDiff->loadLeftFile( leftFile )) {
			logDD("loading file");
			kmw->leftFileChanged();
			logDD("done");
			fileJustLoaded();
		}

	}

} 

void
Kmerge::openRight()
{
	if ( rightFileDir.isEmpty() )
		if ( ! leftFileDir.isEmpty() )
			rightFileDir = leftFileDir.copy();

  	KFileDialog fdg( (const char *) rightFileDir, 0, 
		this, "ropen", true, false );
	fdg.setCaption( i18n( "Open right file") );
	
	if( fdg.exec() && !fdg.selectedFile().isEmpty() ) {
		//logDD("Right file selected: %s", fdg.selectedFile().data());
		
		rightFileDir = fdg.dirPath();
		rightFile = fdg.selectedFile().copy();
		if (! fileDiff->loadRightFile( rightFile )) {
			logDD("loading file");
			kmw->rightFileChanged();
			logDD("done");
			fileJustLoaded();
		}
	}
} 


void 
Kmerge::quit()
{
	//kapp->quit();
}


void 
Kmerge::doDiff()
{
	logDD("running diff");
	if ( diffIO->runDiff(fileDiff)) {
		KMsgBox errDialog( this, i18n("Error"), i18n( "The diff program encountered an error!" ), KMsgBox::EXCLAMATION, "Ok" , "View error");
		int ret = errDialog.exec();  
		logDD("Error dialog returned: %d", ret);
		return;
	}


	kmergeState( KM_DispDiff );
	logDD("calling diffAvailable");
	kmw->diffAvailable();

	/*
	kLineDiff *lineDiff;
	cout << "Results:" << endl;
	char *typeString[] = { "NoDiff", "Added", "Deleted", "Changed" };
	for ( lineDiff = fileDiff->diffList.first(); lineDiff != 0; lineDiff = fileDiff->diffList.next() ) {
		cout << "difference : " << typeString[ lineDiff->type ] << " ";
		cout << lineDiff->leftS << " - " << lineDiff->leftE;
		cout << " , " << lineDiff->rightS << " - " << lineDiff->rightE;
		cout << endl;
	}
	cout << "." << endl;
	*/
		

}


void 
Kmerge::selectFont()
{
  	KFontDialog fontDialog( this, "fontDialog", TRUE  );
	fontDialog.setFont( kmergeCfg->font );
  	fontDialog.show();
	if ( fontDialog.result() ) {
		logDD("Fonts changed!");
		kmergeCfg->font = fontDialog.font( );
		kmw->newDispParam();
	}
}


void Kmerge::selectColors()
{
    	kmColorDg colDg( this );
	int ret = colDg.exec();
	logDD( "color dialog returned %d", ret);
	if (ret == 1) {
		logDD("Colors changed!");
		kmw->newDispParam();
	}
}

void Kmerge::changeDiffOpt()
{
	kmDiffDg dg( this );
	int retValue = dg.exec();
	
	logDD("Diff Options Dialog returned : %d", retValue);
}

void Kmerge::showLineNr()
{
	kmergeCfg->showLineNr = ! kmergeCfg->showLineNr;
	optionPm->setItemChecked( showLineNrId, kmergeCfg->showLineNr );
}

void 
Kmerge::helpContents()
{
}

void
Kmerge::helpIndex()
{
}


void Kmerge::helpAbout()
{
}	
	

void
Kmerge::initConfig()
{
	config = kapp->getConfig();
	config->setGroup("KMerge");
 
	// Color config
	QColor tmpCol = black;
        kmergeCfg->norFgCol = config->readColorEntry( "norFgColor", &tmpCol );
	tmpCol = black;
        kmergeCfg->addFgCol = config->readColorEntry( "addFgColor", &tmpCol );
	tmpCol = black;
	kmergeCfg->chgFgCol = config->readColorEntry( "chgFgColor", &tmpCol );
	tmpCol = black;
	kmergeCfg->delFgCol = config->readColorEntry( "delFgColor", &tmpCol );
	tmpCol = white;
        kmergeCfg->norBgCol = config->readColorEntry( "norBgColor", &tmpCol );
	tmpCol = green;
        kmergeCfg->addBgCol = config->readColorEntry( "addBgColor", &tmpCol );
	tmpCol = yellow;
	kmergeCfg->chgBgCol = config->readColorEntry( "chgBgColor", &tmpCol );
	tmpCol = cyan;
	kmergeCfg->delBgCol = config->readColorEntry( "delBgColor", &tmpCol );

	// font config
	QFont tmpFont( "Courier", 12 );
        kmergeCfg->font = config->readFontEntry( "font", &tmpFont );

	// general config
	kmergeCfg->showLineNr = config->readBoolEntry( "showLineNr", false );
	kmergeCfg->viewDiffOnly = config->readBoolEntry( "viewDiffOnly", false);

        kmergeCfg->ignoreCase = config->readBoolEntry( "ignoreCase", false);
        kmergeCfg->ignoreBlankLines = config->readBoolEntry( "ignoreBlankLines", false);
        kmergeCfg->ignoreAllSpaces = config->readBoolEntry( "ignoreAllSpaces", false);
        kmergeCfg->ignoreSpaces = config->readBoolEntry( "ignoreSpaces", false);
        kmergeCfg->ignoreRegExpLines = config->readBoolEntry( "ignoreRegExpLines", false);
        kmergeCfg->ignoreRegExp = config->readEntry( "ignoreRegExp", "" );
	kmergeCfg->ignoreRegExp.detach();

        kmergeCfg->minimalDifferences = config->readBoolEntry( "minimalDifferences", false);
        kmergeCfg->speedLargeFiles = config->readBoolEntry( "speedLargeFiles", false);

	kmergeCfg->customizeDiff = config->readBoolEntry( "customizeDiff", false);
	kmergeCfg->customExp = config->readEntry( "customExp", "" );
	kmergeCfg->customExp.detach();

}


void Kmerge::writeConfig()
{
	config->setGroup("KMerge");
 
	// Color config
        config->writeEntry( "norFgColor", kmergeCfg->norFgCol );
        config->writeEntry( "addFgColor", kmergeCfg->addFgCol );
	config->writeEntry( "chgFgColor", kmergeCfg->chgFgCol );
	config->writeEntry( "delFgColor", kmergeCfg->delFgCol );
        config->writeEntry( "norBgColor", kmergeCfg->norBgCol );
        config->writeEntry( "addBgColor", kmergeCfg->addBgCol );
	config->writeEntry( "chgBgColor", kmergeCfg->chgBgCol );
	config->writeEntry( "delBgColor", kmergeCfg->delBgCol );

	// font config
	config->writeEntry( "font", kmergeCfg->font );

	// general config
	config->writeEntry( "showLineNr", kmergeCfg->showLineNr);
	config->writeEntry( "viewDiffOnly", kmergeCfg->viewDiffOnly );

        config->writeEntry( "ignoreCase", kmergeCfg->ignoreCase );
        config->writeEntry( "ignoreBlankLines", kmergeCfg->ignoreBlankLines );
        config->writeEntry( "ignoreAllSpaces", kmergeCfg->ignoreAllSpaces );
        config->writeEntry( "ignoreSpaces", kmergeCfg->ignoreSpaces );
        config->writeEntry( "ignoreRegExpLines", kmergeCfg->ignoreRegExpLines );
        config->writeEntry( "ignoreRegExp", kmergeCfg->ignoreRegExp  );
	config->writeEntry( "minimalDifferences", kmergeCfg->minimalDifferences );
        config->writeEntry( "speedLargeFiles", kmergeCfg->speedLargeFiles );

	config->writeEntry( "customizeDiff", kmergeCfg->customizeDiff );
	config->writeEntry( "customExp", kmergeCfg->customExp );

	config->sync();
	
}




/*			--------
 *		 	Menu bar
 *			--------            */
void Kmerge::initMenuBar()
{
	menuBar = KTMainWindow::menuBar();

	// XXX put the accelerators back when ready!
	
	// File Menu
	// 	
	filePm = new QPopupMenu();
	filePm->insertItem( i18n("Open &left ..."),  this, 
		SLOT(openLeft()), CTRL+Key_L, TB_OPENLEFT);
	filePm->insertItem( i18n("Open &right ..."),  this, 
		SLOT(openRight()), CTRL+Key_R, TB_OPENRIGHT);
	filePm->insertSeparator( );
	/*
	filePm->insertItem( i18n("&Quit"),  kapp, SLOT(closeAllWindows())
		 , KAccel::Quit  );
		 */

	menuBar->insertItem( i18n("&File"), filePm );

	// Diffrences Menu
	// 	
	diffPm = new QPopupMenu();
	diffPm->insertItem( i18n("&Analyse differences"),  this, 
			SLOT(doDiff()), 0, TB_DODIFF);
	diffPm->insertItem( i18n("&Previous difference"),  kmw, 
			SLOT(prevDiff()), 0, TB_PREVDIFF );
	diffPm->insertItem( i18n("&Next difference"),  kmw, 
			SLOT(nextDiff()), 0, TB_NEXTDIFF );
	diffPm->insertItem( i18n("&Unselect differences"),  kmw, 
			SLOT(unselectDiff()), 0, TB_UNSELECTDIFF );
	menuBar->insertItem( i18n("&Difference"), diffPm );

	// Merge Menu
	// 	
  	mergePm = new QPopupMenu();
	mergePm->insertItem( i18n("&Merge mode"), this, SLOT(mergeMode()),
			0, TB_MERGEMODE);
	mergePm->insertItem( i18n("&Take left"), kmw, SLOT(leftTakeOne()),
			0, TB_TAKELEFT);
	mergePm->insertItem( i18n("&Take right"), kmw, SLOT(rightTakeOne()),
			0, TB_TAKERIGHT);
	menuBar->insertItem( i18n("&Merge"), mergePm );

	optionPm = new QPopupMenu();
	optionPm->insertItem( i18n("&Font "),  this, SLOT(selectFont()));
	optionPm->insertItem( i18n("&Colors "),  this, SLOT(selectColors()));
        optionPm->insertItem( i18n("&Diff options"), this,
                SLOT(changeDiffOpt()));
	showLineNrId = optionPm->insertItem( i18n("&Show Line Numbers"), this, 
		SLOT(showLineNr()));
	optionPm->setCheckable( TRUE );
	optionPm->setItemChecked( showLineNrId, kmergeCfg->showLineNr );
	optionPm->insertSeparator( );
	optionPm->insertItem( i18n("&Save Options"),  this, 
		SLOT(writeConfig()));
	menuBar->insertItem( i18n("&Options"), optionPm );
	
	helpPm = new QPopupMenu();
	helpPm->insertItem( i18n("Contents ..."), this, SLOT(helpContents())
			/*,*/ );
	helpPm->insertItem( i18n("Index ... "), this, SLOT(helpIndex()));
	helpPm->insertSeparator( );
	helpPm->insertItem( i18n("About ..."), this, SLOT(helpAbout()));
	menuBar->insertItem( i18n("&Help"), helpPm );
}

/*			---------
 *		 	Tool bar
 *			---------                  */
void Kmerge::initToolBar()
{
	toolBar = new KToolBar(this);
  
	// XXX Ensure pixmap path is correctly set	
	//KIconLoader *loader = kapp->getIconLoader();

	QPixmap pm;
	QString pixdir = kapp->kde_datadir() + "/kmerge/";

	pm.load( pixdir + "openleft.xpm" );	
	toolBar->insertButton( pm, TB_OPENLEFT, SIGNAL(clicked()), 
			this, SLOT(openLeft()), TRUE, i18n("Open left") );
	pm.load( pixdir + "openright.xpm" );	
	toolBar->insertButton( pm, TB_OPENRIGHT, SIGNAL(clicked()), 
			this, SLOT(openRight()), TRUE, i18n("Open right") );
	pm.load( pixdir + "dodiff.xpm" );	
	toolBar->insertButton( pm, TB_DODIFF, SIGNAL(clicked()), 
			this, SLOT(doDiff()), TRUE, i18n("Analyses differences"));
	pm.load( pixdir + "prevdiff.xpm" );	
	toolBar->insertButton( pm, TB_PREVDIFF, SIGNAL(clicked()), 
			kmw, SLOT(prevDiff()),TRUE,i18n("Previous difference"));
	pm.load( pixdir + "nextdiff.xpm" );	
	toolBar->insertButton( pm, TB_NEXTDIFF, SIGNAL(clicked()), 
			kmw, SLOT(nextDiff()),TRUE,i18n("Next difference"));

	pm.load( pixdir + "takeLeft.xpm" );	
	toolBar->insertButton( pm, TB_TAKELEFT, SIGNAL(clicked()), 
			kmw, SLOT(leftTakeOne()),TRUE,i18n("Take left"));

	pm.load( pixdir + "takeRight.xpm" );	
	toolBar->insertButton( pm, TB_TAKERIGHT, SIGNAL(clicked()), 
			kmw, SLOT(rightTakeOne()),TRUE,i18n("Take right"));

	// Well, iconLoader is not really necessary here
	/*
	toolBar->insertButton( loader->loadIcon("openleft.xpm"), 
		TB_OPENLEFT, SIGNAL(clicked()), this, SLOT(openLeft()), 
		TRUE, i18n("Open left") );
	toolBar->insertButton( loader->loadIcon("openright.xpm"), 
		TB_OPENRIGHT, SIGNAL(clicked()), this, SLOT(openRight()), 
		TRUE, i18n("Open right") );
	toolBar->insertButton( loader->loadIcon("dodiff.xpm"), 
		TB_DODIFF, SIGNAL(clicked()), this, SLOT(doDiff()), 
		TRUE, i18n("Analyse differences") );
	toolBar->insertButton( loader->loadIcon("prev.xpm"), 
		TB_PREVDIFF, SIGNAL(clicked()), this, SLOT(prevDiff()), 
		TRUE, i18n("Previous difference") );
	toolBar->insertButton( loader->loadIcon("next.xpm"), 
		TB_NEXTDIFF, SIGNAL(clicked()), this, SLOT(nextDiff()), 
		TRUE, i18n("Next difference") );
	*/

	addToolBar(toolBar);
}

/*			----------
 * 			Status bar
 *			----------               */
/*	
void Kmerge::initStatusBar()
{
	statusBar = KTMainWindow::statusBar();
	statusBar->setInsertOrder( KStatusBar::RightToLeft ); 

	KProgress *progressBar = new KProgress( 0, 100, 50, 
					KProgress::Horizontal, statusBar );
	progressBar->setBarStyle( KProgress::Solid );
	progressBar->setTextEnabled( false );
	statusBar->insertWidget( progressBar, progressBar->sizeHint().width(),
		       			SB_PERC );
	statusBar->insertItem( "" , SB_TEXT);
	
}
*/



