/***************************************************************************
                          kmlog.cpp  -  description
                             -------------------
        Version         : $Id: kmlog.cpp,v 1.1 1999/09/23 22:01:56 philcvs Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// System include files
#include <stdarg.h>
#include <unistd.h>
#include <stdio.h>

// Application specific includes
#include "kmlog.h"

static char *kmLogLevelName[] = {
	"FATAL    ", 
	"ERROR    ", 
	"INFO     ",
	"WARNING  ", 
	"DEBUG    "
};

#define LOGMIN LogFatal
#define LOGMAX LogDebug

static FILE * logFile = stderr;
static kmLogLevel curLogLevel = LogDebug;


/** Set file to which all log will be made. If this function is not called
  * before logging, all output will go to stderr
  */
int setLogFile( char * file )
{
	if (logFile != 0 && logFile != stderr) {
		fclose(logFile);
	}
	
	logFile = fopen( file, "w+" );
	if (! logFile) {
		fprintf( stderr, "Could not open log file %s\n", file );
		return -1;
	}
	
	return 0;
}
	

/** Set the log level to 'level', so that every log below level will not
  * be displayed
  */
void setLogLevel( kmLogLevel level)
{
	if (level > LOGMAX)
		level = LOGMAX;
	if (level < LOGMIN)
		level = LOGMIN;
	
	curLogLevel = level;
	logI( "Log level set to %s", kmLogLevelName[level]);
}


/** Log with level */
void logf(kmLogLevel level, char *fmt, ...)
{
	va_list args;

	if ( level > curLogLevel )
		return;

	fprintf(logFile, "%s: ", kmLogLevelName[level] );

	va_start(args, fmt);
	vfprintf(logFile, fmt, args);
	va_end(args);
	fputc('\n', logFile);
	fflush(logFile);
}
