/***************************************************************************
                          kmlog.h  -  description
                             -------------------
        Version         : $Id: kmlog.h,v 1.2 1999/10/20 20:45:59 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMLOG_H 
#define KMLOG_H 
 

enum kmLogLevel {
	LogFatal    =0, 	/* preventing the program to continue */
	LogError    =1, 	/* bug or important problem */
	LogInfo     =2, 	/* information concerning the execution */
	LogWarning  =3, 	/* something strange is going on */
	LogDebug    =4,  	/* debug info */
	LogLowDebug =5  	/* very low debug info */
};

/** Set file to which all log will be made. If this function is not called
  * before logging, all output will go to stderr
  */
int setLogFile( char * file );

/** Set the log level to 'level', so that every log below level will not
  * be displayed. Default is LogDebug (everything log is written out).
  */
void setLogLevel( kmLogLevel level);

/** Log with given level */
void logf(kmLogLevel level, char *fmt, ...);
		
/** Useful macros */
#define logD( format, args... ) logf( LogDebug, 	format , ## args )
#define logI( format, args... ) logf( LogInfo,		format , ## args )
#define logW( format, args... ) logf( LogWarning,	format , ## args )
#define logE( format, args... ) logf( LogError, 	format , ## args )
#define logF( format, args... ) logf( LogFatal, 	format , ## args )

#define log2(loglevel, format, args... ) \
	logf(loglevel, "(l.%d) %s - " format, __LINE__, __FUNCTION__ , ## args )

#define logDD( format, args... ) log2( LogDebug,	format, ## args )
#define logII( format, args... ) log2( LogInfo,		format, ## args )
#define logWW( format, args... ) log2( LogWarning,	format, ## args )
#define logEE( format, args... ) log2( LogError,	format, ## args )
#define logFF( format, args... ) log2( LogFatal,	format, ## args ) 
	
#define kmAssert( test, args... ) {	\
	if (! (test)) {			\
		logWW( args );	\
	}				\
}
                                                           
#endif // KMLOG_H
