/***************************************************************************
                          kmwidget.cpp  -  description
                             -------------------

		The main widget, that handles 2 scrollable areas.

   Version	: $Id: kmwidget.cpp,v 1.10 2000/05/15 09:25:10 philippe Exp $
   Begin	: Tue Sep 21 1999
   Copyright	: (C) 1999 by Philippe Fremy
   E-mail	: pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream.h>

// Qt includes
#include <qstring.h>
#include <qstrlist.h>
#include <qlayout.h>
#include <qframe.h>
#include <qcolor.h>
//#include <qtooltip.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

// Application specific includes
#include "kmlog.h"
#include "kmconfig.h"
#include "kmwidget.h"
#include "mergeDg.h"

#define logLL(args...)


/*
 *                 --------------------
 * 		         kmWidget			     
 *                 --------------------
 */


kmWidget::kmWidget(QWidget *parent, const char *name)
	: QWidget(parent, name)
{
	curDiff = -1;
	fileDiff = 0;
	_merging = false;

	//XXX: use fileDiff to find the names
	//lTitle = new QLabel( "No file", this );
	lTitle = new QLabel( "Left file", this );
	rTitle = new QLabel( "Right file", this );
	//rTitle = new QLabel( "No file", this );
	QFont ft(font());
	ft.setBold( true );
	lTitle->setFont( ft );
	lTitle->setAutoResize( true );
	lTitle->adjustSize();
	lTitle->setMinimumHeight( lTitle->height());
	rTitle->setFont( ft );
	rTitle->setAutoResize( true );
	rTitle->adjustSize();
	rTitle->setMinimumHeight( lTitle->height());

	//QFontMetrics fm(ft);
	//fHeight = fm.height();


	//sbar = new QScrollBar( QScrollBar::Vertical, this );
	//sbar->setRange( 1, 1 );
	
	lsv = new QScrollView( this );
	lsv->setVScrollBarMode( QScrollView::AlwaysOn );
	lsv->setHScrollBarMode( QScrollView::Auto );
	lsv->setContentsPos( 0, 0 );
	lsv->viewport()->setBackgroundColor( kmergeCfg->norBgCol );

	rsv = new QScrollView( this );
	rsv->setVScrollBarMode( QScrollView::AlwaysOn );
	rsv->setHScrollBarMode( QScrollView::Auto );
	rsv->setContentsPos( 0, 0 );
	rsv->viewport()->setBackgroundColor( kmergeCfg->norBgCol );

	lTview = new ktextView( true, lsv->viewport());
	rTview = new ktextView( false, rsv->viewport());
	lsv->addChild( lTview );
	rsv->addChild( rTview );

	//initButtons();

	qme = new QMultiLineEdit( this );
	qme->setReadOnly( true );

	connect( lsv->verticalScrollBar(), SIGNAL(valueChanged(int)),
			this, SLOT(setY(int)) );
	connect( rsv->verticalScrollBar(), SIGNAL(valueChanged(int)),
			this, SLOT(setY(int)) );
	
	initLayout();

}

void kmWidget::leftFileChanged()
{
	lTview->fileChanged();

	if (rTview->dispMode() == DispDiff) {
		rTview->setMode(DispFile);
	}
	updateTview();

	lTitle->setText( fileDiff->leftFileName );
}

void kmWidget::rightFileChanged()
{
	rTview->fileChanged();

	if (lTview->dispMode() == DispDiff) {
		lTview->setMode(DispFile);
	}

	updateTview();

	rTitle->setText( fileDiff->rightFileName );
}

void kmWidget::diffAvailable()
{
	lTview->diffAvailable();
	rTview->diffAvailable();
	updateTview();
}

void kmWidget::updateTview()
{
	lTview->update( lsv->contentsX(), lsv->contentsY(),
			lsv->contentsWidth(), lsv->contentsHeight());
	rTview->update( rsv->contentsX(), rsv->contentsY(),
			rsv->contentsWidth(), rsv->contentsHeight());

}

void kmWidget::newDispParam()
{
	lsv->viewport()->setBackgroundColor( kmergeCfg->norBgCol );
	rsv->viewport()->setBackgroundColor( kmergeCfg->norBgCol );
	lTview->setDispParam();
	lTview->update();
	rTview->setDispParam();
	rTview->update();
}

void kmWidget::setDiff( kFileDiff * _fileDiff)
{
	//XXX: set file names
	fileDiff = _fileDiff;
	lTview->setDiff( fileDiff );
	rTview->setDiff( fileDiff );
}


void kmWidget::unselectDiff()
{
	lTview->selectDiff( -1 );
	rTview->selectDiff( -1 );
	curDiff = -1;
}


void kmWidget::nextDiff()
{
	QRect r;
	
	r = lTview->nextDiff();
	if (! r.isEmpty()) lsv->ensureVisible( lsv->contentsX(), r.y(), lsv->contentsWidth(), r.width());
	r = rTview->nextDiff();
	if (! r.isEmpty()) rsv->ensureVisible( rsv->contentsX(), r.y(), rsv->contentsWidth(), r.width());

}

void kmWidget::prevDiff()
{
	QRect r;
	
	r = lTview->prevDiff();
	if (! r.isEmpty()) lsv->ensureVisible( lsv->contentsX(), r.y(), lsv->contentsWidth(), r.width());
	r = rTview->prevDiff();
	if (! r.isEmpty()) rsv->ensureVisible( rsv->contentsX(), r.y(), rsv->contentsWidth(), r.width());

}

void kmWidget::setMergeMode(bool on)
{
	if (! on) {
		qme->clear();
		qme->hide();
		_merging = false;
		return;
	}

	curDiff = 0;
	_merging = true;
	qme->show();

	takeIdentLines(-1);

	// XXX add a warning
	if (fileDiff->diffList.count() == 0) {
		mergeComplete();
		return;
	}

	QRect r;
	r = lTview->selectDiff(0);
	if (! r.isEmpty()) lsv->ensureVisible( lsv->contentsX(), r.y(), lsv->contentsWidth(), r.width());
	r = rTview->selectDiff(0);
	if (! r.isEmpty()) rsv->ensureVisible( rsv->contentsX(), r.y(), rsv->contentsWidth(), r.width());


}

/** Display identical lines between difference diffNb and diffNb+1.
 * Called with -1 for first difference.
 */
void kmWidget::takeIdentLines(int diffNb)
{
	int i, st, end;

	if (diffNb == -1) st = 1;
	else st = fileDiff->diffList.at(diffNb)->leftE;

	if (diffNb == (int) fileDiff->diffList.count()-1) 
		end = fileDiff->leftStrings.count()+1;
	else 
		end = fileDiff->diffList.at(diffNb+1)->leftS;

	for(i=st; i<end; i++) {
		qme->append( fileDiff->leftStrings.at(i-1));
	}
	qme->setCursorPosition( qme->numLines(), 0 );

}

void kmWidget::leftTakeOne()
{
	int i, st, end;

	st = fileDiff->diffList.at(curDiff)->leftS;
	end = fileDiff->diffList.at(curDiff)->leftE;

	for(i=st; i<end; i++) {
		qme->insertLine( fileDiff->leftStrings.at(i-1));
	}

	takeIdentLines(curDiff);

	nextDiff();
	curDiff++;

	if (curDiff == (int) fileDiff->diffList.count()) 
		mergeComplete();
}

void kmWidget::rightTakeOne()
{
	int i, st, end;

	st = fileDiff->diffList.at(curDiff)->rightS;
	end = fileDiff->diffList.at(curDiff)->rightE;

	for(i=st; i<end; i++) {
		qme->insertLine( fileDiff->rightStrings.at(i-1));
	}

	takeIdentLines(curDiff);

	nextDiff();
	curDiff++;

	if (curDiff == (int) fileDiff->diffList.count()) 
		mergeComplete();
}


void kmWidget::mergeComplete()
{
	curDiff = -1;
	unselectDiff();

    	KmergeOutputDg mergeDg( this, 
			fileDiff->leftFileName, fileDiff->rightFileName,
		       	mergeDir );

	int ret;
	while(1) {
		ret = mergeDg.exec();

		if (ret) {

			// return to dialog if something goes wrong
			if (! saveMerge(mergeDg.fileName)) continue;

			// file is saved now
			break;
		}

		ret = QMessageBox::warning( this, "kmerge", "If you leave now, you will lose the output of your mergings!", "Save output", "Drop");
		if (ret == 1) break;
	}

	emit sg_mergeComplete();
	return;

}


bool kmWidget::saveMerge(QString filename)
{
	QFile f(filename);

	if (! f.open(IO_WriteOnly)) {
		QMessageBox::warning( this, "kmerge", QString("Unable to write file:\n") + filename );
		return false;
	}

	QTextStream ts( &f);
       
	ts << qme->text();

	f.close();

	return true;
}


void kmWidget::leftTakeAll()
{
	nextDiff();
	curDiff++;
	logDD("called");
}

void kmWidget::leftEditOne()
{
	logDD("called");
}

void kmWidget::rightTakeAll()
{
	logDD("called");
}

void kmWidget::rightEditOne()
{
	logDD("called");
}

void kmWidget::bothTakeEdit()
{
	logDD("called");
}


void kmWidget::setY( int y )
{
	if( lsv->contentsY() != y )
		lsv->setContentsPos( lsv->contentsX(), y );
	if( rsv->contentsY() != y )
		rsv->setContentsPos( rsv->contentsX(), y );

}


kmWidget::~kmWidget()
{
}

void kmWidget::initButtons()
{
/*
	// Manages all buttons

	QWidget *butWidget = new QWidget( this );

	_leftTakeAll = new QPushButton( butWidget );
	_leftTakeOne = new QPushButton( butWidget );
	_leftEditOne = new QPushButton( butWidget );
	_rightTakeAll = new QPushButton(butWidget );
	_rightTakeOne = new QPushButton(butWidget );
	_bothTakeEdit = new QPushButton(butWidget );
	_rightEditOne = new QPushButton(butWidget );

	QToolTip::add( _leftTakeAll, "Take all next difference from left file");
	QToolTip::add( _rightTakeAll,"Take all next difference from right file");
	QToolTip::add( _leftTakeOne, "Take that difference from left file");
	QToolTip::add(_rightTakeOne, "Take that difference from right file");
	QToolTip::add( _leftEditOne, "Take and edit that difference from left file");
	QToolTip::add(_rightEditOne, "Take and edit that difference from right file");
	QToolTip::add( _bothTakeEdit, "Edit that difference from left and right file and take it");

	QString s;
       	//s = kapp->kde_datadir() + "kmerge/icons";
	s = "icons/";
	QPixmap pm;

	pm.load(s + "takeAll.xpm");
	_leftTakeAll->setPixmap( pm );
	_rightTakeAll ->setPixmap( pm );
	pm.load( s + "takeOne.xpm" );
	_leftTakeOne->setPixmap( pm );
	_rightTakeOne ->setPixmap( pm );
	pm.load( s + "takeOneEdit.xpm" );
	_leftEditOne ->setPixmap( pm );
	_rightEditOne ->setPixmap( pm );
	pm.load( s + "takeBothEdit.xpm" );
	_bothTakeEdit ->setPixmap( pm );

	_leftTakeAll->setAutoResize(true);
	_leftTakeOne->setAutoResize(true);
	_leftEditOne->setAutoResize(true);
	_rightTakeAll->setAutoResize(true);
	_rightTakeOne->setAutoResize(true);
	_rightEditOne->setAutoResize(true);
	_bothTakeEdit->setAutoResize(true);

	_leftTakeAll->setFixedSize( _leftTakeAll->size() );
	_leftTakeOne->setFixedSize( _leftTakeOne->size() );
	_leftEditOne->setFixedSize( _leftEditOne->size() );
	_rightTakeAll->setFixedSize( _rightTakeAll->size() );
	_rightTakeOne->setFixedSize( _rightTakeOne->size() );
	_rightEditOne->setFixedSize( _rightEditOne->size() );
	_bothTakeEdit->setFixedSize( _bothTakeEdit->size() );

	butWidget->adjustSize();
	butWidget->setMinimumSize( butWidget->size() );

	vly->addWidget( butWidget );
	hly3->addStretch();
	hly3->addWidget( _leftTakeAll );
	hly3->addStretch();
	hly3->addWidget( _leftTakeOne );
	hly3->addStretch();
	hly3->addWidget( _leftEditOne );
	hly3->addStretch();
	hly3->addWidget( _bothTakeEdit );
	hly3->addStretch();
	hly3->addWidget( _rightEditOne );
	hly3->addStretch();
	hly3->addWidget( _rightTakeOne );
	hly3->addStretch();
	hly3->addWidget( _rightTakeAll );
	hly3->addStretch();

	connect( _leftTakeAll, SIGNAL(clicked()), this, SLOT( leftTakeAll()));
	connect( _leftTakeOne, SIGNAL(clicked()), this, SLOT( leftTakeOne()));
	connect( _leftEditOne, SIGNAL(clicked()), this, SLOT( leftEditOne()));
	connect( _rightTakeAll, SIGNAL(clicked()), this, SLOT( rightTakeAll()));
	connect( _rightTakeOne, SIGNAL(clicked()), this, SLOT( rightTakeOne()));
	connect( _rightEditOne, SIGNAL(clicked()), this, SLOT( rightEditOne()));
	connect( _bothTakeEdit, SIGNAL(clicked()), this, SLOT( bothTakeEdit()));
*/
}

void kmWidget::initLayout()
{
	QBoxLayout *vly, *hly1, *hly2;

	vly = new QVBoxLayout( this );
	hly1 = new QHBoxLayout();
	hly2 = new QHBoxLayout();

	vly->addLayout( hly1 );
	hly1->addWidget( lTitle );
	hly1->addWidget( rTitle );

	vly->addLayout( hly2, 2 );
	hly2->addWidget( lsv );
	hly2->addWidget( rsv );

	vly->addWidget( qme, 1 );

	vly->activate();


}




