/***************************************************************************
                          ktextview.cpp  -  description
                             -------------------

		The main widget, that handles 2 scrollable areas.

        Version         : $Id: ktextview.cpp,v 1.8 2000/05/15 09:25:11 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qstring.h>
#include <qstrlist.h>
#include <qcolor.h>
#include <qpainter.h>

// Application specific includes
#include "ktextview.h"
#include "kmlog.h"
#include "diffIO.h"
#include "kmconfig.h"

#define logLL(args...)

extern kmConfig * kmergeCfg;

#define OFFS 10

/*
 *                 -------------------
 * 			kTextView			     
 *                 -------------------
 */
	
ktextView::ktextView( bool _isLeft, QWidget *parent = 0, const char *name = 0)
	: QWidget( parent, name )
{
	xOffs = OFFS;
	lineWidth = -1;
	isLeft = _isLeft;

	dispLines.setAutoDelete( true );
	strings=0;
	fileDiff = 0L;

	curDiff = -1;
	
	setMode( NoDisp );	
	setDispParam();
}
	
ktextView::~ktextView()
{
}

void ktextView::setDiff( kFileDiff * _fileDiff)
{
	fileDiff = _fileDiff;

	bool hasFile;

	if (isLeft) hasFile = ! fileDiff->leftFileName.isEmpty();
	else hasFile = ! fileDiff->rightFileName.isEmpty();

	if (! hasFile) {
		setMode( NoDisp );
		return;
	}

	if (fileDiff->diffList.count()) {
		diffAvailable();
	} else {
		fileChanged();
	}
}

/** Set font, fHeight, strings, lineWidth */
void ktextView::setDispParam()
{
	int ll;
	QFontMetrics fm( kmergeCfg->font );
	ft = kmergeCfg->font;
	fHeight = fm.height();

	setBackgroundColor( kmergeCfg->norBgCol );

	if (fileDiff) {
  	strings = isLeft ? &(fileDiff->leftStrings)
  			: &(fileDiff->rightStrings);
		
  	ll = isLeft ? fileDiff->leftLongestLine
  			     : fileDiff->rightLongestLine;
  	
  	if (strings != 0) {
  		lineWidth =  OFFS + fm.width( strings->at(ll) )
  			     + fm.maxWidth();
  	} else {
  		logDD("strings is empty!!!");
		}
	}

	//update();
}


void ktextView::fileChanged()
{
 	setMode( DispFile );
}

void ktextView::diffAvailable()
{
	setDispParam();

	computeDispLines();

	if (! fileDiff->diffList.count()) {
		setMode( DispFile );
		return;
	}

	setMode(DispDiff);
}

void ktextView::computeDispLines()
{
	logDD("called");
	dispLines.clear();

 	kdispLine *dl=0;
	kLineDiff * lastDiff;
	int i, l, st, end;
	uint u;

	l = 0;
	lastDiff = 0;

	QListIterator<kLineDiff> it( fileDiff->diffList );


	for( it.toFirst(); it.current(); ++it) {
		if (lastDiff == 0) i=0;
		else i = lastDiff->curFileSize + lastDiff->diffSize;

		// Identical lines
		for( ; i<it.current()->curFileSize; i++) {
			dl = new kdispLine;
			dl->type = NoDiff;
			dl->lNum = ++l;
			dispLines.append(dl);
		}

		it.current()->dispLine = dispLines.count();
		
		// Different lines
		if (isLeft) {
			st = it.current()->leftS;
			end = it.current()->leftE;
		} else {
			st = it.current()->rightS;
			end = it.current()->rightE;
		}

		it.current()->dispLine = dispLines.count();
		for(i=st; i<end; i++) {
			dl = new kdispLine;
			dl->type = it.current()->type;
			dl->lNum = ++l;
			dispLines.append(dl);
		}

		// Lines only on one side
		i -= st;
		for(; i<it.current()->diffSize; i++) {
			dl = new kdispLine;
			dl->type = NoDiff;
			dl->lNum = 0;
			dispLines.append(dl);
		}

		lastDiff = it.current();
	}

	if (lastDiff == 0) u=0;
	else u = lastDiff->curFileSize + lastDiff->diffSize;

	// Identical lines
	for( ; u<=strings->count(); u++) {
		dl = new kdispLine;
		dl->type = NoDiff;
		dl->lNum = ++l;
		dispLines.append(dl);
	}




}



const QRect ktextView::curDiffRect()
{
       	int x, y, cx, cy;
       	x=y=cx=cy=0;

	// If no current diff, the whole widget should be repainted
	if (curDiff == -1) return QRect(0,0,width(), height());

	cx = lineWidth;
	y = fHeight * fileDiff->diffList.at(curDiff)->dispLine;
	cy = fHeight * (fileDiff->diffList.at(curDiff)->diffSize);
		
	return QRect(x, y, cx, cy);
}

const QRect ktextView::selectDiff(int diffNb)
{
	if (diffNb == curDiff) return QRect();

	if (diffNb < (int) fileDiff->diffList.count() && diffNb >= -1) {
		update( curDiffRect() );
		curDiff = diffNb;
		QRect r = curDiffRect();
		update(r);
		return r;
	}

	logDD("Bad diff number : %d", diffNb);
	return QRect();
}

const QRect ktextView::prevDiff()
{

	if (curDiff > 0) {

		update( curDiffRect() );
		curDiff--;
		QRect r = curDiffRect();
		update(r);
		return r;
	}

	return QRect();
}

const QRect ktextView::nextDiff()
{
	if (curDiff < (int) fileDiff->diffList.count()-1) {
		update( curDiffRect() );
		curDiff++;
		QRect r = curDiffRect();
		update(r);
		return r;
	}

	return QRect();
}


void ktextView::setMode( DispMode m )
{
	if (mode == m) return;

	mode = m;
	
	switch( mode ) {
	case NoDisp:
		resize(0,0);
		break;
	case DispFile:
		curDiff = -1;
		setDispParam();

		// We disable repaint for the big widget because only the
		// visible part of the widget should be repainted
		setUpdatesEnabled( false );
		resize( lineWidth, strings->count() * fHeight );
		setUpdatesEnabled( true );
		break;
	case DispDiff:
		curDiff = -1;
		//setDispParam();
		// We disable repaint for the big widget because only the
		// visible part of the widget should be repainted
		setUpdatesEnabled( false );
		resize( lineWidth, dispLines.count() * fHeight );
		setUpdatesEnabled( true );
		break;
	}
}

int
ktextView::h2line( int h )
{
	return (int) (h / fHeight);
}




void
ktextView::paintEvent( QPaintEvent * pe )
{
	/*
	logDD("Visible rect : %d-%d x %d-%d", 
		QWidget::visibleRect().left(),
			visibleRect().top(),
			visibleRect().right(),
			visibleRect().bottom());



	logDD("Painting %d-%d x %d-%d", 
			pe->rect().left(),
			pe->rect().top(),
			pe->rect().right(),
			pe->rect().bottom());
			*/

	// We are here to draw something!
	if (mode == NoDisp)
		return;
		
	int curDiffSt=-1, curDiffEnd=-1;

	if (curDiff != -1) {
		if (isLeft) {
			curDiffSt = fileDiff->diffList.at(curDiff)->leftS;
			curDiffEnd = fileDiff->diffList.at(curDiff)->leftE;
		} else {
			curDiffSt = fileDiff->diffList.at(curDiff)->rightS;
			curDiffEnd = fileDiff->diffList.at(curDiff)->rightE;
		}
	}
	
	long cx, cy, cw, ch;
	QPixmap * pm;

	QRect r = pe->rect();
	cw = r.width();
	ch = r.height();
	cx = r.x();
	cy = r.y();	

	//logDD("( QPaintEvent ( %d - %d, %ld x %ld ) )",  r.x(), r.y(), cw, ch ); 
	//logDD(": Deduced: cx = %ld, cy = %ld\n", cx, cy);
	//logDD(": Deduced: cw = %ld, ch = %ld\n", cw, ch);

	int l0, l1, lMax;
	int i, n, h;
	int pmCx, pmCy;
	QColor norBgCol = kmergeCfg->norBgCol;

	// Max lines to display
	if (mode == DispFile)
		lMax = strings->count();
	else
		lMax = dispLines.count();

	l0 = QMIN( h2line(cy), lMax );
	l1 = QMIN( h2line(cy + ch), lMax - 1);
	//logDD(": drawing lines %d - %d", l0, l1);
	
	// clip zone from the pixmap
	pmCx = cx;
	pmCy = (cy - l0 * fHeight);
	//logDD("clip zone from pixmap: %d x %d", pmCx, pmCy);
	
	QColor fgCol, bgCol;
	QFontMetrics fm( ft );

	// draw the text on our private pixmap
	pm = new QPixmap( lineWidth, (l1 - l0 + 1) * fHeight );
	//logDD("size of pixmap: %d x %d",     lineWidth, (l1 - l0 + 1) * fHeight );
	CHECK_PTR( pm );
	QPainter pmQp;
	pmQp.begin( pm );
	pmQp.setFont( ft );
	
	pmQp.setClipRect( pmCx, pmCy, cw, ch );

	// XXX We should never see anything blue!
	//pmQp.fillRect( pmCx, pmCy, cw, ch, blue );

	for (i = l0; i <= l1; i++) {
		
		if (mode == DispFile) {
			h = (i - l0 )  * fHeight;
			pmQp.fillRect( 0, h, lineWidth, h + fHeight, 
					kmergeCfg->norBgCol );

			pmQp.setPen( kmergeCfg->norFgCol );
			pmQp.drawText( 5, h + fm.ascent(), strings->at(i) );
			continue;
		}
		
		// mode is DispDiff
		n = dispLines.at(i)->lNum;
		h = (i - l0 ) * fHeight;

		switch( dispLines.at(i)->type ) {
		case NoDiff:
        		bgCol = kmergeCfg->norBgCol;
			fgCol = kmergeCfg->norFgCol;
			break;
		case Added:
			bgCol = kmergeCfg->addBgCol;
			fgCol = kmergeCfg->addFgCol;
			break;
		case Deleted:
			bgCol = kmergeCfg->delBgCol;
			fgCol = kmergeCfg->delFgCol;
			break;
		case Changed:
			bgCol = kmergeCfg->chgBgCol;
			fgCol = kmergeCfg->chgFgCol;
			break;
		default:
			logEE("displine %d is of unkowntype", i);
		}

		// Selected diff
		if ( (curDiff == -1) || (n >= curDiffSt && n < curDiffEnd)) {
			pmQp.fillRect( 0, h, lineWidth, h + fHeight, bgCol);
		} else {
			pmQp.fillRect( 0, h, xOffs, h + fHeight, bgCol);
			pmQp.fillRect( xOffs, h, 
					lineWidth, h + fHeight, norBgCol);
		}
		
		// If there is no string to draw, continue
		if (n == 0)
			continue;
		
		pmQp.setPen( fgCol );
		pmQp.drawText( 5, h + fm.ascent(), strings->at(n - 1) );

	}
	pmQp.end();
		

	// copy pixmap
	bitBlt( this, cx, cy, pm, pmCx, pmCy, cw, ch /*, CopyROP*/ );

	delete pm;

}








