/***************************************************************************
                          colorDg.cpp  -  description
                             ---------------------
		Class that handles the Option->color dialog.

        Version         : $Id: mergeDg.cpp,v 1.6 2000/05/15 09:23:45 philippe Exp $
        Begin           : Tue Sep 21 1999
        Copyright       : (C) 1999 by Philippe Fremy
        E-mail          : pfremy@chez.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlabel.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

// Kde includes
#include <kapp.h>

// Application specific includes
#include "mergeDg.h"
#include "kmlog.h"

#define logLL logDD


KmergeOutputDg::KmergeOutputDg(QWidget * parent, 
		const QString & _lFile, 
		const QString & _rFile, const QString & mergeDir)
: QDialog( parent, 0, true )
{
	leftName = _lFile;	
	rightName = _rFile;	
	
	setCaption( "Merge output" );


	QBoxLayout * pvly = new QVBoxLayout( this, 5 );
	QBoxLayout * hly;



#define SETSIZE( w ) w->setFixedSize( w->sizeHint()) 

	QFrame * fr = new QFrame( this );
	fr->setFrameStyle( QFrame::Panel | QFrame::Raised);
	pvly->addWidget( fr, 10 );

	QBoxLayout * vly = new QVBoxLayout( fr, 10, 5);

	// Sentence	
	QLabel * labText = new QLabel("Choose merge file output", fr);
	QFont ft(font());
	ft.setBold(true);
	labText->setFont(ft);
	SETSIZE( labText );
	vly->addWidget( labText );
	vly->addSpacing( 10 );

	// File label
	labFile = new QLineEdit( fr );
	//labFile->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labFile->setText( "coucou" );
	labFile->setMinimumSize( labFile->sizeHint());
	labFile->setFixedHeight( labFile->height());
	labFile->setText( "" );

	hly = new QHBoxLayout(5);
	vly->addLayout( hly );

	QPushButton * openBut = new QPushButton( fr );
	QPixmap pm;
	pm.load( kapp->kde_datadir() + "/kmerge/" + "open.xpm" );
	openBut->setPixmap( pm );
	//openBut->adjustSize();
	//openBut->setFixedSize( openBut->size());
	SETSIZE( openBut );
	connect(openBut, SIGNAL(clicked()), this, SLOT(openButPressed()));
	
	hly->addWidget( labFile, 5 );	
	hly->addSpacing( 10 );
	hly->addWidget( openBut );

	//vly->addStretch( 1 );
	vly->addSpacing( 10 );

	hly = new QHBoxLayout(5);
	vly->addLayout( hly );

	QPushButton * leftBut = new QPushButton( "Overwrite left file", fr);
	SETSIZE(leftBut);
	//leftBut->setFixedSize( rightBut->size() );
	connect(leftBut, SIGNAL(clicked()), this, SLOT(leftButPressed()));
	
	// Buttons
	QPushButton * rightBut = new QPushButton( "Overwrite right file", fr );
	SETSIZE( rightBut );
	connect( rightBut, SIGNAL(clicked()), this, SLOT(rightButPressed()));

	hly->addWidget( leftBut );
	hly->addStretch( 1 );
	hly->addWidget( rightBut );

	pvly->addSpacing( 10 );

	QPushButton * cancelBut = new QPushButton( "Cancel", this );
	cancelBut->setMinimumSize(70, 25);
	connect(cancelBut, SIGNAL(clicked()), this, SLOT(cancel()));

	QPushButton * okBut = new QPushButton( "Ok", this );
	okBut->setMinimumSize(70, 25);
	connect(okBut, SIGNAL(clicked()), this, SLOT(ok()));
		
	hly = new QHBoxLayout(5);
	pvly->addLayout( hly );
	hly->addStretch( 1 );
	hly->addWidget( cancelBut, 2 );
	//hly->addSpacing( 10 );
	hly->addStretch( 2 );
	hly->addWidget( okBut, 2 );
	hly->addStretch( 1 );
	
	pvly->activate();
	resize( 400, 50 );
}

KmergeOutputDg::~KmergeOutputDg()
{}

void KmergeOutputDg::openButPressed()
{
	logDD("called");
	QString s;

	if (labFile->text() != leftName || labFile->text() != rightName)  s = labFile->text();
	else s="";

	s = QFileDialog::getSaveFileName( s );
	if (! s.isEmpty()) labFile->setText( s );
}

void KmergeOutputDg::leftButPressed()
{
	logDD("called");
	labFile->setText( leftName );
}

void KmergeOutputDg::rightButPressed()
{
	logDD("called");
	labFile->setText( rightName );
}

void KmergeOutputDg::ok()
{
	logDD("called");

	fileName = labFile->text();
	QFile f(fileName);

	if (f.exists()) {
		int ret = QMessageBox::warning( this, "kmerge", "File exists, overwrite ?", "Yes", "No");
		if (1 == ret) return;
	}

	accept();
}

void KmergeOutputDg::cancel()
{
	logDD("called");
	reject();
}
	


